<?php

declare(strict_types=1);

namespace spec\BitBag\SyliusBonusPointsPlugin\Assigner;

use BitBag\SyliusBonusPointsPlugin\Assigner\BonusPointsAssigner;
use BitBag\SyliusBonusPointsPlugin\Assigner\BonusPointsAssignerInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\EligibleBonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Resolver\CartBonusPointsAmountResolverInterface;
use Doctrine\ORM\EntityManagerInterface;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;

class BonusPointsAssignerSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldImplement(BonusPointsAssignerInterface::class);
    }

    public function let(
        CartBonusPointsAmountResolverInterface $cartBonusPointsAmountResolver,
        FactoryInterface $bonusPointsFactory,
        EntityManagerInterface $bonusPointsEntityManager,
        RepositoryInterface $customerBonusPointsRepository,
        FactoryInterface $customerBonusPointsFactory,
        RepositoryInterface $customerBonusPointsStrategyRepository,
        FactoryInterface $customerBonusPointsStrategyFactory,
        EntityManagerInterface $customerBonusPointsStrategyEntityManager
    )
    {
        $this->beConstructedWith(
            $cartBonusPointsAmountResolver,
            $bonusPointsFactory,
            $bonusPointsEntityManager,
            $customerBonusPointsRepository,
            $customerBonusPointsFactory,
            $customerBonusPointsStrategyRepository,
            $customerBonusPointsStrategyFactory,
            $customerBonusPointsStrategyEntityManager
        );

    }

    function it_does_nothing_when_the_cart_has_no_bonus_points(
        OrderInterface $order,
        CartBonusPointsAmountResolverInterface $cartBonusPointsAmountResolver,
        EntityManagerInterface $bonusPointsEntityManager,
        CustomerInterface $customer
    )
    {
        $order->getCustomer()->willReturn($customer);
        $cartBonusPointsAmountResolver->getEligibleBonusPointsStrategies($order)->willReturn([]);
        $cartBonusPointsAmountResolver->sumEligibleBonusPointsStrategiesTotal([])->willReturn(0);
        $bonusPointsEntityManager->flush()->shouldNotBeCalled();

        $this->assign($order);
    }

    function it_creates_the_bonus_points_when_there_are_any_in_the_cart(
        OrderInterface $order,
        CustomerInterface $customer,
        CartBonusPointsAmountResolverInterface $cartBonusPointsAmountResolver,
        FactoryInterface $bonusPointsFactory,
        BonusPointsInterface $bonusPoints,
        EntityManagerInterface $bonusPointsEntityManager,
        FactoryInterface $customerBonusPointsFactory,
        CustomerBonusPointsInterface $customerBonusPoints
    )
    {
        $order
            ->getCustomer()
            ->willReturn($customer);

        $cartBonusPointsAmountResolver
            ->getEligibleBonusPointsStrategies($order)
            ->willReturn([]);

        $cartBonusPointsAmountResolver
            ->sumEligibleBonusPointsStrategiesTotal([])
            ->willReturn(100);

        $bonusPointsFactory
            ->createNew()
            ->willReturn($bonusPoints);

        $customerBonusPointsFactory
            ->createNew()
            ->willReturn($customerBonusPoints);

        $bonusPointsEntityManager
            ->persist($bonusPoints)
            ->shouldBeCalledOnce();

        $bonusPointsEntityManager
            ->flush()
            ->shouldBeCalledOnce();

        $this->assign($order);
    }

    function it_finds_the_customer_bonus_points_entry_if_there_are_any(
        OrderInterface $order,
        CustomerInterface $customer,
        CartBonusPointsAmountResolverInterface $cartBonusPointsAmountResolver,
        FactoryInterface $bonusPointsFactory,
        BonusPointsInterface $bonusPoints,
        RepositoryInterface $customerBonusPointsRepository,
        CustomerBonusPointsInterface $customerBonusPoints
    )
    {
        $order
            ->getCustomer()
            ->willReturn($customer);

        $cartBonusPointsAmountResolver
            ->getEligibleBonusPointsStrategies($order)
            ->willReturn([]);

        $cartBonusPointsAmountResolver
            ->sumEligibleBonusPointsStrategiesTotal([])
            ->willReturn(100);

        $bonusPointsFactory
            ->createNew()
            ->willReturn($bonusPoints);

        $customerBonusPointsRepository->findOneBy([
            'customer' => $customer
        ])->willReturn($customerBonusPoints)->shouldBeCalledOnce();

        $customerBonusPoints->getId()->willReturn(Argument::any());
        $customerBonusPoints->addBonusPoints($bonusPoints)->shouldBeCalledOnce();
        $customerBonusPointsRepository->add($customerBonusPoints)->shouldNotBeCalled();

        $this->assign($order);
    }

    function it_creates_the_customer_bonus_points_entry_if_there_are_none(
        OrderInterface $order,
        CustomerInterface $customer,
        CartBonusPointsAmountResolverInterface $cartBonusPointsAmountResolver,
        FactoryInterface $bonusPointsFactory,
        BonusPointsInterface $bonusPoints,
        RepositoryInterface $customerBonusPointsRepository,
        CustomerBonusPointsInterface $customerBonusPoints,
        FactoryInterface $customerBonusPointsFactory
    )
    {
        $order
            ->getCustomer()
            ->willReturn($customer);

        $cartBonusPointsAmountResolver
            ->getEligibleBonusPointsStrategies($order)
            ->willReturn([]);

        $cartBonusPointsAmountResolver
            ->sumEligibleBonusPointsStrategiesTotal([])
            ->willReturn(100);

        $bonusPointsFactory
            ->createNew()
            ->willReturn($bonusPoints);

        $customerBonusPointsRepository->findOneBy([
            'customer' => $customer
        ])->willReturn(null)->shouldBeCalledOnce();

        $customerBonusPointsFactory->createNew()->willReturn($customerBonusPoints)->shouldBeCalledOnce();

        $customerBonusPoints->getId()->willReturn(null);
        $customerBonusPoints->setCustomer($customer)->shouldBeCalledOnce();
        $customerBonusPointsRepository->add($customerBonusPoints)->shouldBeCalledOnce();
        $customerBonusPoints->addBonusPoints($bonusPoints)->shouldBeCalledOnce();

        $this->assign($order);
    }

    function it_adds_a_customer_bonus_points_strategy_entry_for_all_eligible_bonus_points_strategies(
        OrderInterface $order,
        CustomerInterface $customer,
        CartBonusPointsAmountResolverInterface $cartBonusPointsAmountResolver,
        FactoryInterface $bonusPointsFactory,
        BonusPointsInterface $bonusPoints,
        RepositoryInterface $customerBonusPointsRepository,
        CustomerBonusPointsInterface $customerBonusPoints,
        EligibleBonusPointsStrategyInterface $eligibleBonusPointsStrategy,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        RepositoryInterface $customerBonusPointsStrategyRepository,
        FactoryInterface $customerBonusPointsStrategyFactory,
        CustomerBonusPointsStrategyInterface $customerBonusPointsStrategy,
        EntityManagerInterface $customerBonusPointsStrategyEntityManager
    )
    {
        $order
            ->getCustomer()
            ->willReturn($customer);

        $cartBonusPointsAmountResolver
            ->getEligibleBonusPointsStrategies($order)
            ->willReturn([$eligibleBonusPointsStrategy]);

        $cartBonusPointsAmountResolver
            ->sumEligibleBonusPointsStrategiesTotal([$eligibleBonusPointsStrategy])
            ->willReturn(100);

        $eligibleBonusPointsStrategy
            ->getBonusPointsStrategy()
            ->willReturn($bonusPointsStrategy);

        $eligibleBonusPointsStrategy
            ->getBonusPointsAmount()
            ->willReturn(100);

        $bonusPointsFactory
            ->createNew()
            ->willReturn($bonusPoints);

        $customerBonusPointsRepository->findOneBy([
            'customer' => $customer
        ])->willReturn($customerBonusPoints)->shouldBeCalledOnce();

        $customerBonusPoints->getId()->willReturn(Argument::any());
        $customerBonusPoints->addBonusPoints($bonusPoints)->shouldBeCalledOnce();

        $customerBonusPointsStrategyRepository
            ->findOneBy([
                'customer' => $customer,
                'bonusPointsStrategy' => $bonusPointsStrategy,
            ])
            ->willReturn(null)
            ->shouldBeCalledOnce();

        $customerBonusPointsStrategyFactory
            ->createNew()
            ->willReturn($customerBonusPointsStrategy)
            ->shouldBeCalledOnce();

        $customerBonusPointsStrategy
            ->setCustomer($customer)
            ->shouldBeCalledOnce();

        $customerBonusPointsStrategy
            ->setBonusPointsStrategy($bonusPointsStrategy)
            ->shouldBeCalledOnce();

        $customerBonusPointsStrategyEntityManager
            ->persist($customerBonusPointsStrategy)
            ->shouldBeCalledOnce();

        $customerBonusPointsStrategyEntityManager
            ->flush()
            ->shouldBeCalledOnce();

        $this->assign($order);
    }

    function it_does_not_add_a_customer_bonus_points_strategy_entry_for_a_strategy_already_applied_for_the_customer(
        OrderInterface $order,
        CustomerInterface $customer,
        CartBonusPointsAmountResolverInterface $cartBonusPointsAmountResolver,
        FactoryInterface $bonusPointsFactory,
        BonusPointsInterface $bonusPoints,
        RepositoryInterface $customerBonusPointsRepository,
        CustomerBonusPointsInterface $customerBonusPoints,
        EligibleBonusPointsStrategyInterface $eligibleBonusPointsStrategy,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        RepositoryInterface $customerBonusPointsStrategyRepository,
        FactoryInterface $customerBonusPointsStrategyFactory,
        CustomerBonusPointsStrategyInterface $customerBonusPointsStrategy,
        EntityManagerInterface $customerBonusPointsStrategyEntityManager
    )
    {
        $order
            ->getCustomer()
            ->willReturn($customer);

        $cartBonusPointsAmountResolver
            ->getEligibleBonusPointsStrategies($order)
            ->willReturn([$eligibleBonusPointsStrategy]);

        $cartBonusPointsAmountResolver
            ->sumEligibleBonusPointsStrategiesTotal([$eligibleBonusPointsStrategy])
            ->willReturn(100);

        $eligibleBonusPointsStrategy
            ->getBonusPointsStrategy()
            ->willReturn($bonusPointsStrategy);

        $eligibleBonusPointsStrategy
            ->getBonusPointsAmount()
            ->willReturn(100);

        $bonusPointsFactory
            ->createNew()
            ->willReturn($bonusPoints);

        $customerBonusPointsRepository->findOneBy([
            'customer' => $customer
        ])->willReturn($customerBonusPoints)->shouldBeCalledOnce();

        $customerBonusPoints->getId()->willReturn(Argument::any());
        $customerBonusPoints->addBonusPoints($bonusPoints)->shouldBeCalledOnce();

        $customerBonusPointsStrategyRepository
            ->findOneBy([
                'customer' => $customer,
                'bonusPointsStrategy' => $bonusPointsStrategy,
            ])
            ->willReturn($customerBonusPointsStrategy)
            ->shouldBeCalledOnce();

        $customerBonusPointsStrategyFactory
            ->createNew()
            ->shouldNotBeCalled();

        $customerBonusPointsStrategyEntityManager
            ->persist(Argument::any())
            ->shouldNotBeCalled();

        $customerBonusPointsStrategyEntityManager
            ->flush()
            ->shouldNotBeCalled();

        $this->assign($order);
    }
}
