<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Checker\Rule;

use BitBag\SyliusBonusPointsPlugin\Checker\Rule\AppliedOnceRuleChecker;
use BitBag\SyliusBonusPointsPlugin\Checker\Rule\BonusPointsStrategyRuleCheckerInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsStrategyInterface;
use InvalidArgumentException;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItemInterface;
use Sylius\Component\Core\Model\ProductInterface;
use Sylius\Component\Promotion\Exception\UnsupportedTypeException;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Sylius\Component\Taxonomy\Repository\TaxonRepositoryInterface;

class AppliedOnceRuleCheckerSpec extends ObjectBehavior
{
    function it_is_a_bonus_points_strategy_rule()
    {
        $this->shouldImplement(BonusPointsStrategyRuleCheckerInterface::class);
    }

    function let(
        RepositoryInterface $bonusPointsStrategyRepository,
        RepositoryInterface $customerBonusPointsStrategyRepository
    )
    {
        $this->beConstructedWith($bonusPointsStrategyRepository, $customerBonusPointsStrategyRepository);
    }

    function it_returns_true_if_there_is_no_customer_for_the_order(
        OrderItemInterface $orderItem,
        OrderInterface $order
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getCustomer()->willReturn(null);

        $this
            ->isEligible($orderItem, ['strategyCode' => ''])
            ->shouldReturn(true);
    }

    function it_finds_the_strategy_code_configured_by_its_strategy_code(
        OrderItemInterface $orderItem,
        OrderInterface $order,
        CustomerInterface $customer,
        RepositoryInterface $bonusPointsStrategyRepository,
        BonusPointsStrategyInterface $bonusPointsStrategy
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getCustomer()->willReturn($customer);

        $bonusPointsStrategyRepository
            ->findOneBy(['code' => 'foo'])
            ->willReturn($bonusPointsStrategy)
            ->shouldBeCalledOnce();

        $this
            ->isEligible($orderItem, ['strategyCode' => 'foo']);
    }

    function it_returns_false_if_the_strategy_has_been_applied_once_for_the_customer(
        OrderItemInterface $orderItem,
        OrderInterface $order,
        CustomerInterface $customer,
        RepositoryInterface $bonusPointsStrategyRepository,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        RepositoryInterface $customerBonusPointsStrategyRepository,
        CustomerBonusPointsStrategyInterface $customerBonusPointsStrategy
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getCustomer()->willReturn($customer);

        $bonusPointsStrategyRepository
            ->findOneBy(['code' => 'foo'])
            ->willReturn($bonusPointsStrategy);

        $customerBonusPointsStrategyRepository
            ->findOneBy([
                'customer' => $customer,
                'bonusPointsStrategy' => $bonusPointsStrategy
            ])
            ->willReturn($customerBonusPointsStrategy)
            ->shouldBeCalledOnce();

        $this
            ->isEligible($orderItem, ['strategyCode' => 'foo'])
            ->shouldReturn(false);
    }

    function it_returns_true_if_the_strategy_has_not_been_applied_for_the_customer(
        OrderItemInterface $orderItem,
        OrderInterface $order,
        CustomerInterface $customer,
        RepositoryInterface $bonusPointsStrategyRepository,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        RepositoryInterface $customerBonusPointsStrategyRepository
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getCustomer()->willReturn($customer);

        $bonusPointsStrategyRepository
            ->findOneBy(['code' => 'foo'])
            ->willReturn($bonusPointsStrategy);

        $customerBonusPointsStrategyRepository
            ->findOneBy([
                'customer' => $customer,
                'bonusPointsStrategy' => $bonusPointsStrategy
            ])
            ->willReturn(null)
            ->shouldBeCalledOnce();

        $this
            ->isEligible($orderItem, ['strategyCode' => 'foo'])
            ->shouldReturn(true);
    }

    function it_throws_an_invalid_argument_exception_if_the_configuration_has_no_strategy_code(
        OrderItemInterface $orderItem
    )
    {
        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('isEligible', [$orderItem, []]);
    }

    function it_throws_an_invalid_argument_exception_if_the_order_item_is_not_attached_to_an_order(
        OrderItemInterface $orderItem
    )
    {
        $orderItem->getOrder()->willReturn(null);

        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('isEligible', [$orderItem, ['strategyCode' => '']]);
    }

    function it_throw_an_invalid_argument_exception_if_the_configured_strategy_is_not_found(
        OrderItemInterface $orderItem,
        OrderInterface $order,
        CustomerInterface $customer,
        RepositoryInterface $bonusPointsStrategyRepository
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getCustomer()->willReturn($customer);

        $bonusPointsStrategyRepository->findOneBy(['code' => 'foo'])->willReturn(null);

        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('isEligible', [$orderItem, ['strategyCode' => 'foo']]);
    }
}
