<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\EventListener;

use BitBag\SyliusBonusPointsPlugin\Calculator\DelegatingBonusPointsStrategyCalculatorInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\EventListener\BonusPointsEventListener;
use BitBag\SyliusBonusPointsPlugin\Repository\BonusPointsStrategyRepositoryInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityManagerInterface;
use Mockery\Matcher\Any;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Component\EventDispatcher\GenericEvent;

class BonusPointsEventListenerSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(BonusPointsEventListener::class);
    }

    function let(
        BonusPointsStrategyRepositoryInterface $bonusPointsStrategyRepository,
        DelegatingBonusPointsStrategyCalculatorInterface $delegatingBonusPointsStrategyCalculator,
        FactoryInterface $bonusPointsFactory,
        EntityManagerInterface $bonusPointsEntityManager,
        RepositoryInterface $customerBonusPointsRepository,
        FactoryInterface $customerBonusPointsFactory,
        GenericEvent $event,
        Any $subject
    )
    {
        $event->getSubject()->willReturn($subject);

        $this->beConstructedWith(
            $bonusPointsStrategyRepository,
            $delegatingBonusPointsStrategyCalculator,
            $bonusPointsFactory,
            $bonusPointsEntityManager,
            $customerBonusPointsRepository,
            $customerBonusPointsFactory
        );
    }

    function it_uses_the_delegating_bonus_points_strategy_calculator_to_calculate_the_bonus_points_earned(
        BonusPointsStrategyRepositoryInterface $bonusPointsStrategyRepository,
        DelegatingBonusPointsStrategyCalculatorInterface $delegatingBonusPointsStrategyCalculator,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        GenericEvent $event,
        Any $subject
    )
    {
        $bonusPointsStrategyRepository
            ->findBy(['event' => 'foo'])
            ->willReturn(new ArrayCollection([$bonusPointsStrategy->getWrappedObject()]));

        $delegatingBonusPointsStrategyCalculator->calculate($subject, $bonusPointsStrategy, 0)->shouldBeCalledOnce();

        $this($event, 'foo');
    }

    function it_applies_the_bonus_points_strategies_matching_the_event_name(
        BonusPointsStrategyRepositoryInterface $bonusPointsStrategyRepository,
        DelegatingBonusPointsStrategyCalculatorInterface $delegatingBonusPointsStrategyCalculator,
        BonusPointsStrategyInterface $firstBonusPointsStrategy,
        BonusPointsStrategyInterface $secondBonusPointsStrategy,
        GenericEvent $event,
        Any $subject
    )
    {
        $bonusPointsStrategyRepository
            ->findBy(['event' => 'foo'])
            ->willReturn(new ArrayCollection([$firstBonusPointsStrategy->getWrappedObject(), $secondBonusPointsStrategy->getWrappedObject()]));

        $delegatingBonusPointsStrategyCalculator->calculate($subject, $firstBonusPointsStrategy, 0)->shouldBeCalledOnce();
        $delegatingBonusPointsStrategyCalculator->calculate($subject, $secondBonusPointsStrategy, 0)->shouldBeCalledOnce();

        $this($event, 'foo');
    }

    function it_does_nothing_if_there_is_no_strategies_matching_the_event_name(
        BonusPointsStrategyRepositoryInterface $bonusPointsStrategyRepository,
        FactoryInterface $bonusPointsFactory,
        EntityManagerInterface $bonusPointsEntityManager,
        GenericEvent $event
    )
    {
        $bonusPointsStrategyRepository
            ->findBy(['event' => 'foo'])
            ->willReturn(new ArrayCollection());

        $bonusPointsFactory->createNew()->shouldNotBeCalled();
        $bonusPointsEntityManager->persist(Argument::any())->shouldNotBeCalled();
        $bonusPointsEntityManager->flush()->shouldNotBeCalled();

        $this($event, 'foo');
    }

    function it_does_nothing_if_there_is_no_bonus_points_calculated(
        BonusPointsStrategyRepositoryInterface $bonusPointsStrategyRepository,
        DelegatingBonusPointsStrategyCalculatorInterface $delegatingBonusPointsStrategyCalculator,
        FactoryInterface $bonusPointsFactory,
        EntityManagerInterface $bonusPointsEntityManager,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        GenericEvent $event,
        Any $subject
    )
    {
        $bonusPointsStrategyRepository
            ->findBy(['event' => 'foo'])
            ->willReturn(new ArrayCollection([$bonusPointsStrategy->getWrappedObject()]));

        $delegatingBonusPointsStrategyCalculator->calculate($subject, $bonusPointsStrategy, 0)->willReturn(0);

        $bonusPointsFactory->createNew()->shouldNotBeCalled();
        $bonusPointsEntityManager->persist(Argument::any())->shouldNotBeCalled();
        $bonusPointsEntityManager->flush()->shouldNotBeCalled();

        $this($event, 'foo');
    }

    function it_save_a_bonus_points_entry_for_the_existing_customer_bonus_points_entry_if_there_is_bonus_points_calculated(
        BonusPointsStrategyRepositoryInterface $bonusPointsStrategyRepository,
        DelegatingBonusPointsStrategyCalculatorInterface $delegatingBonusPointsStrategyCalculator,
        FactoryInterface $bonusPointsFactory,
        EntityManagerInterface $bonusPointsEntityManager,
        RepositoryInterface $customerBonusPointsRepository,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        BonusPointsInterface $bonusPoints,
        GenericEvent $event,
        CustomerInterface $subject,
        CustomerBonusPointsInterface $customerBonusPoints
    )
    {
        $bonusPointsStrategyRepository
            ->findBy(['event' => 'foo'])
            ->willReturn(new ArrayCollection([$bonusPointsStrategy->getWrappedObject()]));

        $delegatingBonusPointsStrategyCalculator->calculate($subject, $bonusPointsStrategy, 0)->willReturn(150);

        $bonusPointsFactory->createNew()->willReturn($bonusPoints)->shouldBeCalledOnce();

        $customerBonusPointsRepository->findOneBy([
            'customer' => $subject
        ])->willReturn($customerBonusPoints);

        $customerBonusPoints->getId()->willReturn(Argument::any());
        $customerBonusPoints->addBonusPoints($bonusPoints)->shouldBeCalledOnce();

        $bonusPointsEntityManager->persist($bonusPoints)->shouldBeCalledOnce();
        $bonusPointsEntityManager->flush()->shouldBeCalledOnce();

        $this($event, 'foo');
    }

    function it_save_a_bonus_points_entry_and_a_customer_bonus_points_entry_if_there_is_bonus_points_calculated_and_the_customer_had_no_bonus_points(
        BonusPointsStrategyRepositoryInterface $bonusPointsStrategyRepository,
        DelegatingBonusPointsStrategyCalculatorInterface $delegatingBonusPointsStrategyCalculator,
        FactoryInterface $bonusPointsFactory,
        EntityManagerInterface $bonusPointsEntityManager,
        RepositoryInterface $customerBonusPointsRepository,
        FactoryInterface $customerBonusPointsFactory,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        BonusPointsInterface $bonusPoints,
        GenericEvent $event,
        CustomerInterface $subject,
        CustomerBonusPointsInterface $customerBonusPoints
    )
    {
        $bonusPointsStrategyRepository
            ->findBy(['event' => 'foo'])
            ->willReturn(new ArrayCollection([$bonusPointsStrategy->getWrappedObject()]));

        $delegatingBonusPointsStrategyCalculator->calculate($subject, $bonusPointsStrategy, 0)->willReturn(150);

        $bonusPointsFactory->createNew()->willReturn($bonusPoints)->shouldBeCalledOnce();

        $customerBonusPointsRepository->findOneBy([
            'customer' => $subject
        ])->willReturn(null);

        $customerBonusPointsFactory->createNew()->willReturn($customerBonusPoints)->shouldBeCalledOnce();

        $customerBonusPoints->getId()->willReturn(null);
        $customerBonusPoints->setCustomer($subject)->shouldBeCalledOnce();
        $customerBonusPointsRepository->add($customerBonusPoints)->shouldBeCalledOnce();

        $customerBonusPoints->addBonusPoints($bonusPoints)->shouldBeCalledOnce();

        $bonusPointsEntityManager->persist($bonusPoints)->shouldBeCalledOnce();
        $bonusPointsEntityManager->flush()->shouldBeCalledOnce();

        $this($event, 'foo');
    }
}
