<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\EventListener;

use BitBag\SyliusBonusPointsPlugin\Context\CustomerBonusPointsContextInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsAwareInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\EventListener\OrderBonusPointsListener;
use InvalidArgumentException;
use Mockery\Matcher\Any;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Sylius\Bundle\ResourceBundle\Doctrine\ORM\EntityRepository;
use Sylius\Bundle\ResourceBundle\Event\ResourceControllerEvent;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;

class OrderBonusPointsListenerSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(OrderBonusPointsListener::class);
    }

    function let(
        EntityRepository $bonusPointsRepository,
        FactoryInterface $bonusPointsFactory,
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        ResourceControllerEvent $resourceControllerEvent,
        Any $subject
    )
    {
        $resourceControllerEvent->getSubject()->willReturn($subject);
        $this->beConstructedWith(
            $bonusPointsRepository,
            $bonusPointsFactory,
            $customerBonusPointsContext
        );
    }

    function it_throws_an_invalid_argument_exception_if_the_subject_is_not_an_orderd(
        ResourceControllerEvent $resourceControllerEvent
    )
    {
        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('assignBonusPoints', [$resourceControllerEvent]);
    }

    function it_throws_an_invalid_argument_exception_if_the_subject_is_not_an_bonus_points_aware(
        ResourceControllerEvent $resourceControllerEvent,
        Any $subject
    )
    {
        $subject->implement(OrderInterface::class);

        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('assignBonusPoints', [$resourceControllerEvent]);
    }

    function it_does_nothing_if_the_order_has_no_bonus_points(
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        ResourceControllerEvent $resourceControllerEvent,
        Any $subject
    )
    {
        $subject->implement(OrderInterface::class);
        $subject->implement(BonusPointsAwareInterface::class);

        $subject->getBonusPoints()->willReturn(null);

        $customerBonusPointsContext->getCustomerBonusPoints()->shouldNotBeCalled();

        $this->assignBonusPoints($resourceControllerEvent);
    }

    function it_does_nothing_if_the_customer_has_no_bonus_points(
        EntityRepository $bonusPointsRepository,
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        ResourceControllerEvent $resourceControllerEvent,
        Any $subject
    )
    {
        $subject->implement(OrderInterface::class);
        $subject->implement(BonusPointsAwareInterface::class);

        $subject->getBonusPoints()->willReturn(150);
        $customerBonusPointsContext->getCustomerBonusPoints()->willReturn(null);

        $bonusPointsRepository->findOneBy(Argument::any())->shouldNotBeCalled();

        $this->assignBonusPoints($resourceControllerEvent);
    }

    function it_creates_a_used_bonus_points_entry_if_there_is_none_for_the_order(
        EntityRepository $bonusPointsRepository,
        FactoryInterface $bonusPointsFactory,
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        ResourceControllerEvent $resourceControllerEvent,
        Any $subject,
        CustomerBonusPointsInterface $customerBonusPoints,
        BonusPointsInterface $bonusPoints
    )
    {
        $subject->implement(OrderInterface::class);
        $subject->implement(BonusPointsAwareInterface::class);

        $subject->getBonusPoints()->willReturn(150);
        $customerBonusPointsContext->getCustomerBonusPoints()->willReturn($customerBonusPoints);

        $bonusPointsRepository->findOneBy(['order' => $subject, 'isUsed' => true])->willReturn(null);

        $customerBonusPoints->addBonusPointsUsed($bonusPoints)->shouldBeCalledOnce();
        $bonusPointsFactory->createNew()->willReturn($bonusPoints)->shouldBeCalledOnce();
        $bonusPointsRepository->add($bonusPoints)->shouldBeCalledOnce();

        $this->assignBonusPoints($resourceControllerEvent);
    }

    function it_binds_the_used_bonus_points_to_the_customer(
        EntityRepository $bonusPointsRepository,
        FactoryInterface $bonusPointsFactory,
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        ResourceControllerEvent $resourceControllerEvent,
        Any $subject,
        CustomerBonusPointsInterface $customerBonusPoints,
        BonusPointsInterface $bonusPoints
    )
    {
        $subject->implement(OrderInterface::class);
        $subject->implement(BonusPointsAwareInterface::class);

        $subject->getBonusPoints()->willReturn(150);
        $customerBonusPointsContext->getCustomerBonusPoints()->willReturn($customerBonusPoints);

        $bonusPointsRepository->findOneBy(['order' => $subject, 'isUsed' => true])->willReturn($bonusPoints);

        $customerBonusPoints->addBonusPointsUsed($bonusPoints)->shouldBeCalledOnce();

        $bonusPointsRepository->add($bonusPoints)->shouldBeCalledOnce();

        $this->assignBonusPoints($resourceControllerEvent);
    }
}
