<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Resolver;

use BitBag\SyliusBonusPointsPlugin\Context\CustomerBonusPointsContextInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\Resolver\BonusPointsResolverInterface;
use Doctrine\Common\Collections\ArrayCollection;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Core\Model\OrderInterface;

class BonusPointsResolverSpec extends ObjectBehavior
{
    function it_is_a_bonus_points_resolver()
    {
        $this->shouldImplement(BonusPointsResolverInterface::class);
    }

    function let(
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        BonusPointsInterface $notUsedBonusPoints,
        BonusPointsInterface $expiredBonusPoints,
        BonusPointsInterface $usedBonusPointsForAnotherOrder,
        BonusPointsInterface $usedBonusPointsForTheGivenOrder,
        OrderInterface $givenOrder,
        OrderInterface $anotherOrder
    )
    {
        $notUsedBonusPoints->getPoints()->willReturn(333);
        $notUsedBonusPoints->getExpiresAt()->willReturn((new \DateTime())->modify('+1 year'));

        $expiredBonusPoints->getPoints()->willReturn(150)->shouldNotBeCalled();
        $expiredBonusPoints->getExpiresAt()->willReturn((new \DateTime())->modify('-1 year'));

        $usedBonusPointsForAnotherOrder->getPoints()->willReturn(231);
        $usedBonusPointsForAnotherOrder->getOrder()->willReturn($anotherOrder);

        $usedBonusPointsForTheGivenOrder->getPoints()->willReturn(52);
        $usedBonusPointsForTheGivenOrder->getOrder()->willReturn($givenOrder);

        $this->beConstructedWith($customerBonusPointsContext);
    }

    function it_returns_zero_if_the_customer_has_no_bonus_points(
        CustomerBonusPointsContextInterface $customerBonusPointsContext
    )
    {
        $customerBonusPointsContext->getCustomerBonusPoints()->willReturn(null);

        $this->resolveBonusPoints()->shouldReturn(0);
    }

    function it_returns_zero_if_the_customer_has_no_bonus_points_that_are_not_expired(
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        CustomerBonusPointsInterface $customerBonusPoints,
        BonusPointsInterface $expiredBonusPoints
    )
    {
        $customerBonusPointsContext->getCustomerBonusPoints()->willReturn($customerBonusPoints);

        $customerBonusPoints
            ->getBonusPoints()
            ->willReturn(new ArrayCollection([$expiredBonusPoints->getWrappedObject()]));

        $this->resolveBonusPoints()->shouldReturn(0);
    }

    function it_resolves_the_customer_bonus_points_that_are_not_expired(
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        CustomerBonusPointsInterface $customerBonusPoints,
        BonusPointsInterface $expiredBonusPoints,
        BonusPointsInterface $notUsedBonusPoints
    )
    {
        $customerBonusPointsContext->getCustomerBonusPoints()->willReturn($customerBonusPoints);

        $customerBonusPoints
            ->getBonusPoints()
            ->willReturn(new ArrayCollection([$expiredBonusPoints->getWrappedObject(), $notUsedBonusPoints->getWrappedObject()]));

        $customerBonusPoints
            ->getBonusPointsUsed()
            ->willReturn(new ArrayCollection());

        $this->resolveBonusPoints()->shouldReturn(333);
    }

    function it_resolves_the_customer_bonus_points_that_are_not_used_or_expired(
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        CustomerBonusPointsInterface $customerBonusPoints,
        BonusPointsInterface $expiredBonusPoints,
        BonusPointsInterface $notUsedBonusPoints,
        BonusPointsInterface $usedBonusPointsForAnotherOrder
    )
    {
        $customerBonusPointsContext->getCustomerBonusPoints()->willReturn($customerBonusPoints);

        $customerBonusPoints
            ->getBonusPoints()
            ->willReturn(new ArrayCollection([$expiredBonusPoints->getWrappedObject(), $notUsedBonusPoints->getWrappedObject()]));

        $customerBonusPoints
            ->getBonusPointsUsed()
            ->willReturn(new ArrayCollection([$usedBonusPointsForAnotherOrder->getWrappedObject()]));

        $this->resolveBonusPoints()->shouldReturn(102);
    }

    function it_resolves_the_customer_bonus_points_that_are_available_including_those_used_on_the_given_order(
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        CustomerBonusPointsInterface $customerBonusPoints,
        BonusPointsInterface $expiredBonusPoints,
        BonusPointsInterface $notUsedBonusPoints,
        BonusPointsInterface $usedBonusPointsForAnotherOrder,
        BonusPointsInterface $usedBonusPointsForTheGivenOrder,
        OrderInterface $givenOrder
    )
    {
        $customerBonusPointsContext->getCustomerBonusPoints()->willReturn($customerBonusPoints);

        $customerBonusPoints
            ->getBonusPoints()
            ->willReturn(new ArrayCollection([$expiredBonusPoints->getWrappedObject(), $notUsedBonusPoints->getWrappedObject()]));

        $customerBonusPoints
            ->getBonusPointsUsed()
            ->willReturn(new ArrayCollection([$usedBonusPointsForAnotherOrder->getWrappedObject(), $usedBonusPointsForTheGivenOrder->getWrappedObject()]));

        $this->resolveBonusPoints($givenOrder)->shouldReturn(102);
    }

    function it_returns_zero_if_the_customer_has_used_more_points_than_currently_available(
        CustomerBonusPointsContextInterface $customerBonusPointsContext,
        CustomerBonusPointsInterface $customerBonusPoints,
        BonusPointsInterface $expiredBonusPoints,
        BonusPointsInterface $notUsedBonusPoints,
        BonusPointsInterface $usedBonusPointsForAnotherOrder,
        BonusPointsInterface $usedBonusPointsForTheGivenOrder,
        OrderInterface $givenOrder
    )
    {
        $customerBonusPointsContext->getCustomerBonusPoints()->willReturn($customerBonusPoints);

        $customerBonusPoints
            ->getBonusPoints()
            ->willReturn(new ArrayCollection([$expiredBonusPoints->getWrappedObject(), $notUsedBonusPoints->getWrappedObject()]));

        $usedBonusPointsForAnotherOrder->getPoints()->willReturn(999999);

        $customerBonusPoints
            ->getBonusPointsUsed()
            ->willReturn(new ArrayCollection([$usedBonusPointsForAnotherOrder->getWrappedObject(), $usedBonusPointsForTheGivenOrder->getWrappedObject()]));

        $this->resolveBonusPoints()->shouldReturn(0);
    }
}
