<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Validator\Constraints;

use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsAwareInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\Resolver\BonusPointsResolverInterface;
use BitBag\SyliusBonusPointsPlugin\Validator\Constraints\BonusPointsAvailability;
use InvalidArgumentException;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Context\ExecutionContextInterface;
use Symfony\Component\Validator\Violation\ConstraintViolationBuilderInterface;

class BonusPointsAvailabilityValidatorSpec extends ObjectBehavior
{
    function it_is_a_constraint_validator()
    {
        $this->shouldHaveType(ConstraintValidator::class);
    }

    function let(
        BonusPointsResolverInterface $bonusPointsResolver,
        RepositoryInterface $bonusPointsRepository,
        ExecutionContextInterface $context
    )
    {
        $this->beConstructedWith($bonusPointsResolver, $bonusPointsRepository);
        $this->initialize($context);
    }

    function it_throws_an_invalid_argument_exception_if_the_order_is_not_bonus_points_aware(
        OrderInterface $order,
        Constraint $constraint
    )
    {
        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('validate', [$order, $constraint]);
    }

    function it_throws_an_invalid_argument_exception_if_the_constraint_is_not_a_bonus_points_availability_constraint(
        OrderInterface $order,
        Constraint $constraint
    )
    {
        $order->implement(BonusPointsAwareInterface::class);
        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('validate', [$order, $constraint]);
    }

    function it_uses_the_points_on_the_order_if_there_are_any(
        OrderInterface $order,
        BonusPointsResolverInterface $bonusPointsResolver,
        RepositoryInterface $bonusPointsRepository
    )
    {
        $order->implement(BonusPointsAwareInterface::class);
        $constraint = new BonusPointsAvailability();

        $order
            ->getBonusPoints()
            ->willReturn(10)
            ->shouldBeCalledTimes(2);

        $bonusPointsRepository
            ->findOneBy([Argument::any()])
            ->shouldNotBeCalled();

        $bonusPointsResolver
            ->resolveBonusPoints($order)
            ->willReturn(50);

        $this
            ->validate($order, $constraint);
    }

    function it_uses_the_repository_to_find_the_used_points_on_the_order_if_there_are_none_on_the_order(
        OrderInterface $order,
        BonusPointsResolverInterface $bonusPointsResolver,
        RepositoryInterface $bonusPointsRepository,
        BonusPointsInterface $bonusPoints
    )
    {
        $order->implement(BonusPointsAwareInterface::class);
        $constraint = new BonusPointsAvailability();

        $order
            ->getBonusPoints()
            ->willReturn(null)
            ->shouldBeCalledOnce();

        $bonusPointsRepository
            ->findOneBy([
                'order' => $order,
                'isUsed' => true
            ])
            ->shouldBeCalledOnce()
            ->willReturn($bonusPoints);

        $bonusPoints
            ->getPoints()
            ->willReturn(10)
            ->shouldBeCalledOnce();

        $bonusPointsResolver
            ->resolveBonusPoints($order)
            ->willReturn(50);

        $this
            ->validate($order, $constraint);
    }

    function it_does_nothing_if_the_amount_of_bonus_points_is_lower_than_the_number_of_points_available_for_the_customer_including_those_used_on_the_given_order(
        OrderInterface $order,
        BonusPointsResolverInterface $bonusPointsResolver,
        ExecutionContextInterface $context
    )
    {
        $order->implement(BonusPointsAwareInterface::class);
        $constraint = new BonusPointsAvailability();

        $order
            ->getBonusPoints()
            ->willReturn(10)
            ->shouldBeCalledTimes(2);

        $bonusPointsResolver
            ->resolveBonusPoints($order)
            ->willReturn(50)
            ->shouldBeCalledOnce();

        $context
            ->buildViolation(Argument::any())
            ->shouldNotBeCalled();

        $this
            ->validate($order, $constraint);
    }

    function it_does_nothing_if_the_amount_of_bonus_points_is_equal_to_the_number_of_points_available_for_the_customer_including_those_used_on_the_given_order(
        OrderInterface $order,
        BonusPointsResolverInterface $bonusPointsResolver,
        ExecutionContextInterface $context
    )
    {
        $order->implement(BonusPointsAwareInterface::class);
        $constraint = new BonusPointsAvailability();

        $order
            ->getBonusPoints()
            ->willReturn(10)
            ->shouldBeCalledTimes(2);

        $bonusPointsResolver
            ->resolveBonusPoints($order)
            ->willReturn(10)
            ->shouldBeCalledOnce();

        $context
            ->buildViolation(Argument::any())
            ->shouldNotBeCalled();

        $this
            ->validate($order, $constraint);
    }

    function it_builds_a_violation_if_the_amount_of_bonus_points_is_greater_to_the_number_of_points_available_for_the_customer_including_those_used_on_the_given_order(
        OrderInterface $order,
        BonusPointsResolverInterface $bonusPointsResolver,
        ConstraintViolationBuilderInterface $constraintViolationBuilder,
        ExecutionContextInterface $context
    )
    {
        $order->implement(BonusPointsAwareInterface::class);
        $constraint = new BonusPointsAvailability();
        $constraint->message = 'Lorem ipsum dolor sit amet';

        $order
            ->getBonusPoints()
            ->willReturn(50)
            ->shouldBeCalledTimes(2);

        $bonusPointsResolver
            ->resolveBonusPoints($order)
            ->willReturn(10)
            ->shouldBeCalledOnce();

        $context
            ->buildViolation('Lorem ipsum dolor sit amet')
            ->willReturn($constraintViolationBuilder)
            ->shouldBeCalledOnce();

        $constraintViolationBuilder->addViolation()->shouldBeCalledOnce();


        $this
            ->validate($order, $constraint);
    }
}
